@@ -4,16 +4,20 @@ import time
 from threading import Thread
 from pychess.System import fident
 
-def repeat (func, *args, **kwargs):
+
+def repeat(func, *args, **kwargs):
     """ Repeats a function in a new thread until it returns False """
-    def run ():
+
+    def run():
         while func(*args, **kwargs):
             pass
-    t = Thread(target=run, name=fident(func))
-    t.daemon = True
-    t.start()
 
-def repeat_sleep (func, sleeptime, recur=False):
+    thread = Thread(target=run, name=fident(func))
+    thread.daemon = True
+    thread.start()
+
+
+def repeat_sleep(func, sleeptime, recur=False):
     """
     Runs func in a thread and repeats it approximately each sleeptime [s]
     until func returns False. Notice that we sleep first, then run. Not the
@@ -22,11 +26,12 @@ def repeat_sleep (func, sleeptime, recur=False):
     argument has to be optional, as it wont be used first time, and it has
     to be non-None.
     """
-    def run ():
+
+    def run():
         last = time.time()
         val = None
         while True:
-            time.sleep(time.time()-last + sleeptime)
+            time.sleep(time.time() - last + sleeptime)
             if not time:
                 # If python has been shutdown while we were sleeping, the
                 # imported modules will be None
@@ -34,8 +39,11 @@ def repeat_sleep (func, sleeptime, recur=False):
             last = time.time()
             if recur and val:
                 val = func(val)
-            else: val = func()
-            if not val: break
-    t = Thread(target=run, name=fident(func))
-    t.daemon = True
-    t.start()
+            else:
+                val = func()
+            if not val:
+                break
+
+    thread = Thread(target=run, name=fident(func))
+    thread.daemon = True
+    thread.start()
